@@ -61,7 +61,6 @@ typedef struct TrackerDBCursorSqliteClass TrackerDBCursorSqliteClass;
 typedef struct TrackerDBStatementSqlite      TrackerDBStatementSqlite;
 typedef struct TrackerDBStatementSqliteClass TrackerDBStatementSqliteClass;
 
-
 struct TrackerDBCursorSqlite {
 	GObject parent_instance;
 	TrackerDBCursorSqlitePrivate *priv;
@@ -76,13 +75,13 @@ struct TrackerDBInterfaceSqlitePrivate {
 	sqlite3 *db;
 
 	GHashTable *dynamic_statements;
-	GHashTable *statements;
 
 	GSList *function_data;
 
 	guint in_transaction : 1;
 	guint ro : 1;
 	guint fts_initialized : 1;
+	volatile gint interrupt;
 };
 
 struct TrackerDBStatementSqlitePrivate {
@@ -244,15 +243,18 @@ function_sparql_uri_is_parent (sqlite3_context *context,
 		return;
 	}
 
+	parent_len = sqlite3_value_bytes (argv[0]);
+
 	/* Check only one argument, it's going to
 	 * be compared with the other anyway.
 	 */
-	if (strstr (parent, "://") == NULL) {
-		sqlite3_result_int (context, FALSE);
-		return;
-	}
 
-	parent_len = strlen (parent);
+	if (!(parent_len >= 7 && (parent[4] == ':' && parent[5] == '/' && parent[6] == '/'))) {
+		if (strstr (parent, "://") == NULL) {
+			sqlite3_result_int (context, FALSE);
+			return;
+		}
+	}
 
 	/* Remove trailing '/', will
 	 * be checked later on uri.
@@ -313,15 +315,18 @@ function_sparql_uri_is_descendant (sqlite3_context *context,
 		return;
 	}
 
+	parent_len = sqlite3_value_bytes (argv[0]);
+
 	/* Check only one argument, it's going to
 	 * be compared with the other anyway.
 	 */
-	if (strstr (parent, "://") == NULL) {
-		sqlite3_result_int (context, FALSE);
-		return;
-	}
 
-	parent_len = strlen (parent);
+	if (!(parent_len >= 7 && (parent[4] == ':' && parent[5] == '/' && parent[6] == '/'))) {
+		if (strstr (parent, "://") == NULL) {
+			sqlite3_result_int (context, FALSE);
+			return;
+		}
+	}
 
 	/* Remove trailing '/', will
 	 * be checked later on uri.
@@ -483,6 +488,12 @@ function_sparql_regex (sqlite3_context *context,
 	sqlite3_result_int (context, ret);
 }
 
+static int
+check_interrupt (void *user_data)
+{
+	TrackerDBInterfaceSqlitePrivate *priv = user_data;
+	return g_atomic_int_get (&priv->interrupt);
+}
 
 static void
 open_database (TrackerDBInterfaceSqlitePrivate *priv)
@@ -503,8 +514,11 @@ open_database (TrackerDBInterfaceSqlitePrivate *priv)
 		}
 	}
 
+	sqlite3_progress_handler (priv->db, 100,
+	                          check_interrupt, priv);
+
 	sqlite3_create_function (priv->db, "SparqlRegex", 3, SQLITE_ANY,
-	                         priv, &function_sparql_regex, 
+	                         priv, &function_sparql_regex,
 	                         NULL, NULL);
 
 	sqlite3_create_function (priv->db, "SparqlHaversineDistance", 4, SQLITE_ANY,
@@ -604,24 +618,22 @@ tracker_db_interface_sqlite_get_property (GObject    *object,
 }
 
 static void
-close_database (TrackerDBInterfaceSqlitePrivate *priv)
+close_database (GObject                         *object,
+                TrackerDBInterfaceSqlitePrivate *priv)
 {
 	gint rc;
 
-	if (priv->fts_initialized) {
-		tracker_fts_shutdown ();
-	}
-
 	g_hash_table_unref (priv->dynamic_statements);
 	priv->dynamic_statements = NULL;
 
-	g_hash_table_unref (priv->statements);
-	priv->statements = NULL;
-
 	g_slist_foreach (priv->function_data, (GFunc) g_free, NULL);
 	g_slist_free (priv->function_data);
 	priv->function_data = NULL;
 
+	if (priv->fts_initialized) {
+		tracker_fts_shutdown (object);
+	}
+
 	rc = sqlite3_close (priv->db);
 	g_warn_if_fail (rc == SQLITE_OK);
 }
@@ -634,7 +646,7 @@ tracker_db_interface_sqlite_fts_init (TrackerDBInterfaceSqlite *interface,
 
 	priv = TRACKER_DB_INTERFACE_SQLITE_GET_PRIVATE (interface);
 
-	tracker_fts_init (priv->db, create);
+	tracker_fts_init (priv->db, create, G_OBJECT (interface));
 	priv->fts_initialized = TRUE;
 }
 
@@ -645,10 +657,11 @@ tracker_db_interface_sqlite_finalize (GObject *object)
 
 	priv = TRACKER_DB_INTERFACE_SQLITE_GET_PRIVATE (object);
 
-	close_database (priv);
+	close_database (object, priv);
 
 	g_message ("Closed sqlite3 database:'%s'", priv->filename);
 
+
 	g_free (priv->filename);
 
 	G_OBJECT_CLASS (tracker_db_interface_sqlite_parent_class)->finalize (object);
@@ -694,10 +707,6 @@ prepare_database (TrackerDBInterfaceSqlitePrivate *priv)
 	priv->dynamic_statements = g_hash_table_new_full (g_str_hash, g_str_equal,
 	                                                  (GDestroyNotify) g_free,
 	                                                  (GDestroyNotify) g_object_unref);
-	priv->statements = g_hash_table_new_full (g_str_hash, g_str_equal,
-	                                          (GDestroyNotify) g_free,
-	                                          (GDestroyNotify) sqlite3_finalize);
-
 }
 
 static void
@@ -757,8 +766,9 @@ add_row (TrackerDBResultSet *result_set,
 
 
 static TrackerDBStatement *
-tracker_db_interface_sqlite_create_statement (TrackerDBInterface *db_interface,
-                                              const gchar        *query)
+tracker_db_interface_sqlite_create_statement (TrackerDBInterface  *db_interface,
+                                              GError             **error,
+                                              const gchar         *query)
 {
 	TrackerDBInterfaceSqlitePrivate *priv;
 	TrackerDBStatementSqlite *stmt;
@@ -769,11 +779,27 @@ tracker_db_interface_sqlite_create_statement (TrackerDBInterface *db_interface,
 
 	if (!stmt) {
 		sqlite3_stmt *sqlite_stmt;
+		int retval;
 
 		g_debug ("Preparing query: '%s'", query);
 
-		if (sqlite3_prepare_v2 (priv->db, query, -1, &sqlite_stmt, NULL) != SQLITE_OK) {
-			g_critical ("Unable to prepare query '%s': %s", query, sqlite3_errmsg (priv->db));
+		retval = sqlite3_prepare_v2 (priv->db, query, -1, &sqlite_stmt, NULL);
+
+		if (retval != SQLITE_OK) {
+
+			if (retval == SQLITE_INTERRUPT) {
+				g_set_error (error,
+				             TRACKER_DB_INTERFACE_ERROR,
+				             TRACKER_DB_INTERRUPTED,
+				             "Interrupted");
+			} else {
+				g_set_error (error,
+				             TRACKER_DB_INTERFACE_ERROR,
+				             TRACKER_DB_QUERY_ERROR,
+				             "%s",
+				             sqlite3_errmsg (priv->db));
+			}
+
 			return NULL;
 		}
 
@@ -786,13 +812,6 @@ tracker_db_interface_sqlite_create_statement (TrackerDBInterface *db_interface,
 	return g_object_ref (stmt);
 }
 
-static void
-foreach_print_error (gpointer key, gpointer value, gpointer stmt)
-{
-	if (value == stmt)
-		g_print ("In %s\n", (char*) key);
-}
-
 static TrackerDBResultSet *
 create_result_set_from_stmt (TrackerDBInterfaceSqlite  *interface,
                              sqlite3_stmt              *stmt,
@@ -809,7 +828,12 @@ create_result_set_from_stmt (TrackerDBInterfaceSqlite  *interface,
 	while (result == SQLITE_OK  ||
 	       result == SQLITE_ROW) {
 
-		result = sqlite3_step (stmt);
+		if (g_atomic_int_get (&priv->interrupt) == 1) {
+			result = SQLITE_INTERRUPT;
+			sqlite3_reset (stmt);
+		} else {
+			result = sqlite3_step (stmt);
+		}
 
 		switch (result) {
 		case SQLITE_ERROR:
@@ -828,7 +852,6 @@ create_result_set_from_stmt (TrackerDBInterfaceSqlite  *interface,
 	}
 
 	if (result != SQLITE_DONE) {
-		g_hash_table_foreach (priv->statements, foreach_print_error, stmt);
 
 		/* This is rather fatal */
 		if (sqlite3_errcode (priv->db) == SQLITE_IOERR ||
@@ -855,11 +878,18 @@ create_result_set_from_stmt (TrackerDBInterfaceSqlite  *interface,
 			           sqlite3_errcode (priv->db),
 			           sqlite3_errmsg (priv->db));
 		} else {
-			g_set_error (error,
-			             TRACKER_DB_INTERFACE_ERROR,
-			             TRACKER_DB_QUERY_ERROR,
-			             "%s",
-			             sqlite3_errmsg (priv->db));
+			if (result == SQLITE_INTERRUPT) {
+				g_set_error (error,
+				             TRACKER_DB_INTERFACE_ERROR,
+				             TRACKER_DB_INTERRUPTED,
+				             "Interrupted");
+			} else {
+				g_set_error (error,
+				             TRACKER_DB_INTERFACE_ERROR,
+				             TRACKER_DB_QUERY_ERROR,
+				             "%s",
+				             sqlite3_errmsg (priv->db));
+			}
 		}
 
 		/* If there was an error, result set may be invalid or incomplete */
@@ -918,17 +948,27 @@ tracker_db_interface_sqlite_interrupt (TrackerDBInterface *iface)
 	TrackerDBInterfaceSqlitePrivate *priv;
 
 	priv = TRACKER_DB_INTERFACE_SQLITE_GET_PRIVATE (iface);
-	sqlite3_interrupt (priv->db);
+
+	g_atomic_int_set (&priv->interrupt, 1);
 
 	return TRUE;
 }
 
+static void
+tracker_db_interface_sqlite_reset_interrupt (TrackerDBInterface *iface)
+{
+	TrackerDBInterfaceSqlitePrivate *priv;
+	priv = TRACKER_DB_INTERFACE_SQLITE_GET_PRIVATE (iface);
+	priv->interrupt = 0;
+}
+
 static void
 tracker_db_interface_sqlite_iface_init (TrackerDBInterfaceIface *iface)
 {
 	iface->create_statement = tracker_db_interface_sqlite_create_statement;
 	iface->execute_query = tracker_db_interface_sqlite_execute_query;
 	iface->interrupt = tracker_db_interface_sqlite_interrupt;
+	iface->reset_interrupt = tracker_db_interface_sqlite_reset_interrupt;
 }
 
 TrackerDBInterface *
@@ -1179,13 +1219,19 @@ static gboolean
 tracker_db_cursor_sqlite_iter_next (TrackerDBCursor *cursor,
                                     GError         **error)
 {
-	TrackerDBCursorSqlitePrivate *priv;
-	priv = TRACKER_DB_CURSOR_SQLITE_GET_PRIVATE (cursor);
+	TrackerDBCursorSqlitePrivate *priv = TRACKER_DB_CURSOR_SQLITE_GET_PRIVATE (cursor);
+	TrackerDBStatementSqlite *stmt = priv->ref_stmt;
+	TrackerDBStatementSqlitePrivate *stmt_priv = TRACKER_DB_STATEMENT_SQLITE_GET_PRIVATE (stmt);
+	TrackerDBInterfaceSqlite *iface = stmt_priv->db_interface;
+	TrackerDBInterfaceSqlitePrivate *iface_priv = TRACKER_DB_INTERFACE_SQLITE_GET_PRIVATE (iface);
 
 	if (!priv->finished) {
-		guint result = SQLITE_BUSY;
+		guint result;
 
-		while (result == SQLITE_BUSY || result == SQLITE_IOERR_BLOCKED) {
+		if (g_atomic_int_get (&iface_priv->interrupt) == 1) {
+			result = SQLITE_INTERRUPT;
+			sqlite3_reset (priv->stmt);
+		} else {
 			result = sqlite3_step (priv->stmt);
 		}
 
@@ -1195,10 +1241,6 @@ tracker_db_cursor_sqlite_iter_next (TrackerDBCursor *cursor,
 			             TRACKER_DB_INTERRUPTED,
 			             "Interrupted");
 		} else if (result != SQLITE_ROW && result != SQLITE_DONE) {
-			TrackerDBStatementSqlite *stmt = priv->ref_stmt;
-			TrackerDBStatementSqlitePrivate *stmt_priv = TRACKER_DB_STATEMENT_SQLITE_GET_PRIVATE (stmt);
-			TrackerDBInterfaceSqlite *iface = stmt_priv->db_interface;
-			TrackerDBInterfaceSqlitePrivate *iface_priv = TRACKER_DB_INTERFACE_SQLITE_GET_PRIVATE (iface);
 
 			g_set_error (error,
 			             TRACKER_DB_INTERFACE_ERROR,
