@@ -235,6 +235,7 @@ ensure_result_set_state (TrackerDBResultSet *result_set)
 
 TrackerDBStatement *
 tracker_db_interface_create_statement (TrackerDBInterface  *interface,
+                                       GError             **error,
                                        const gchar         *query,
                                        ...)
 {
@@ -254,7 +255,7 @@ tracker_db_interface_create_statement (TrackerDBInterface  *interface,
 	va_end (args);
 
 	iface = TRACKER_DB_INTERFACE_GET_IFACE (interface);
-	stmt = iface->create_statement (interface, str);
+	stmt = iface->create_statement (interface, error, str);
 	g_free (str);
 
 	return stmt;
@@ -325,6 +326,21 @@ tracker_db_interface_interrupt (TrackerDBInterface *interface)
 	return TRACKER_DB_INTERFACE_GET_IFACE (interface)->interrupt (interface);
 }
 
+void
+tracker_db_interface_reset_interrupt (TrackerDBInterface *interface)
+{
+	g_return_if_fail (TRACKER_IS_DB_INTERFACE (interface));
+
+	if (!TRACKER_DB_INTERFACE_GET_IFACE (interface)->reset_interrupt) {
+		g_critical ("Database abstraction %s doesn't implement "
+		            "the method reset_interrupt()",
+		            G_OBJECT_TYPE_NAME (interface));
+		return;
+	}
+
+	TRACKER_DB_INTERFACE_GET_IFACE (interface)->reset_interrupt (interface);
+}
+
 gboolean
 tracker_db_interface_start_transaction (TrackerDBInterface *interface)
 {
