@@ -251,42 +251,6 @@ read_outline (PDFDoc               *document,
 	}
 }
 
-static void
-insert_keywords (TrackerSparqlBuilder *metadata,
-                 gchar                *keywords)
-{
-	char *saveptr, *p;
-	size_t len;
-
-	p = keywords;
-	keywords = strchr (keywords, '"');
-
-	if (keywords) {
-		keywords++;
-	} else {
-		keywords = p;
-	}
-
-	len = strlen (keywords);
-	if (keywords[len - 1] == '"') {
-		keywords[len - 1] = '\0';
-	}
-
-	for (p = strtok_r (keywords, ",;", &saveptr);
-	     p;
-	     p = strtok_r (NULL, ",;", &saveptr)) {
-		tracker_sparql_builder_predicate (metadata, "nao:hasTag");
-
-		tracker_sparql_builder_object_blank_open (metadata);
-		tracker_sparql_builder_predicate (metadata, "a");
-		tracker_sparql_builder_object (metadata, "nao:Tag");
-
-		tracker_sparql_builder_predicate (metadata, "nao:prefLabel");
-		tracker_sparql_builder_object_unvalidated (metadata, p);
-
-		tracker_sparql_builder_object_blank_close (metadata);
-	}
-}
 
 static void
 page_get_size (Page    *page,
@@ -314,26 +278,30 @@ page_get_size (Page    *page,
 
 static gchar *
 extract_content (PDFDoc *document,
-                 guint   n_words)
+                 gsize   n_bytes)
 {
 	Page *page;
 	Catalog *catalog;
 	GString *string;
-	gint n_pages, i, words;
-	gchar *t;
+	gint n_pages, i;
+	gsize n_bytes_remaining;
+	GTimer *timer;
 
 	n_pages = document->getNumPages();
 	string = g_string_new ("");
-	words = i = 0;
+	i = 0;
+	n_bytes_remaining = n_bytes;
 	catalog = document->getCatalog();
 
-	while (i < n_pages && words < n_words) {
-		guint normalized_words = 0;
+	timer = g_timer_new ();
+
+	while (i < n_pages && n_bytes_remaining > 0 && g_timer_elapsed (timer, NULL) < 5) {
 		Gfx *gfx;
 		GooString *sel_text;
 		TextOutputDev *text_dev;
 		PDFRectangle pdf_selection;
 		gdouble height = 0, width = 0;
+		gsize len_to_validate;
 
 		page = catalog->getPage (i + 1);
 		i++;
@@ -360,23 +328,32 @@ extract_content (PDFDoc *document,
 
 		sel_text = text_dev->getSelectionText (&pdf_selection, selectionStyleWord);
 
-		t = tracker_text_normalize (sel_text->getCString (), n_words - words, &normalized_words);
+		len_to_validate = MIN (n_bytes_remaining, strlen (sel_text->getCString ()));
 
-		words += normalized_words;
-		g_string_append (string, t);
+		if (tracker_text_validate_utf8 (sel_text->getCString (),
+		                                len_to_validate,
+		                                &string,
+		                                NULL)) {
+			/* A whitespace is added to separate next strings appended */
+			g_string_append_c (string, ' ');
+		}
 
-		g_free (t);
+		/* Update accumulated UTF-8 bytes read */
+		n_bytes_remaining -= len_to_validate;
 
 		delete gfx;
 		delete text_dev;
 	}
 
+	g_timer_destroy (timer);
+
 	return g_string_free (string, FALSE);
 }
 
 static void
 write_pdf_data (PDFData               data,
-                TrackerSparqlBuilder *metadata)
+                TrackerSparqlBuilder *metadata,
+                GPtrArray            *keywords)
 {
 	if (!tracker_is_empty_string (data.title)) {
 		tracker_sparql_builder_predicate (metadata, "nie:title");
@@ -404,7 +381,7 @@ write_pdf_data (PDFData               data,
 	}
 
 	if (!tracker_is_empty_string (data.keywords)) {
-		insert_keywords (metadata, data.keywords);
+		tracker_keywords_parse (keywords, data.keywords);
 	}
 }
 
@@ -494,15 +471,17 @@ extract_pdf (const gchar          *uri,
              TrackerSparqlBuilder *preupdate,
              TrackerSparqlBuilder *metadata)
 {
-	TrackerFTSConfig *fts_config;
+	TrackerConfig *config;
 	TrackerXmpData *xd = NULL;
 	PDFData pd = { 0 }; /* actual data */
 	PDFData md = { 0 }; /* for merging */
 	PDFDoc *document;
 	gchar *content;
-	guint n_words;
+	gsize n_bytes;
 	Object obj;
 	Catalog *catalog;
+	GPtrArray *keywords;
+	guint i;
 
 	g_type_init ();
 
@@ -549,15 +528,19 @@ extract_pdf (const gchar          *uri,
 
 	document->getDocInfo (&obj);
 	if (obj.isDict ()) {
+		gchar *creation_date;
 		Dict *info_dict = obj.getDict();
 		pd.title = info_dict_get_string (info_dict, "Title");
 		pd.author = info_dict_get_string (info_dict, "Author");
 		pd.subject = info_dict_get_string (info_dict, "Subject");
 		pd.keywords = info_dict_get_string (info_dict, "Keywords");
-		pd.creation_date = info_dict_get_string (info_dict, "CreationDate");
+		creation_date = info_dict_get_string (info_dict, "CreationDate");
+		pd.creation_date = tracker_date_guess (creation_date);
+		g_free (creation_date);
 	}
 	obj.free ();
 
+	keywords = g_ptr_array_new ();
 
 	catalog = document->getCatalog ();
 	if (catalog && catalog->isOk ()) {
@@ -580,14 +563,14 @@ extract_pdf (const gchar          *uri,
 			md.date = (gchar *) tracker_coalesce_strip (3, pd.creation_date, xd->date, xd->time_original);
 			md.author = (gchar *) tracker_coalesce_strip (2, pd.author, xd->creator);
 
-			write_pdf_data (md, metadata);
+			write_pdf_data (md, metadata, keywords);
 
 			if (xd->keywords) {
-				insert_keywords (metadata, xd->keywords);
+				tracker_keywords_parse (keywords, xd->keywords);
 			}
 
 			if (xd->pdf_keywords) {
-				insert_keywords (metadata, xd->pdf_keywords);
+				tracker_keywords_parse (keywords, xd->pdf_keywords);
 			}
 
 			if (xd->publisher) {
@@ -777,15 +760,32 @@ extract_pdf (const gchar          *uri,
 		/* So if we are here we have NO XMP data and we just
 		 * write what we know from Poppler.
 		 */
-		write_pdf_data (pd, metadata);
+		write_pdf_data (pd, metadata, keywords);
 	}
 
+	for (i = 0; i < keywords->len; i++) {
+		gchar *p;
+
+		p = (gchar *) g_ptr_array_index (keywords, i);
+
+		tracker_sparql_builder_predicate (metadata, "nao:hasTag");
+
+		tracker_sparql_builder_object_blank_open (metadata);
+		tracker_sparql_builder_predicate (metadata, "a");
+		tracker_sparql_builder_object (metadata, "nao:Tag");
+		tracker_sparql_builder_predicate (metadata, "nao:prefLabel");
+		tracker_sparql_builder_object_unvalidated (metadata, p);
+		tracker_sparql_builder_object_blank_close (metadata);
+		g_free (p);
+	}
+	g_ptr_array_free (keywords, TRUE);
+
 	tracker_sparql_builder_predicate (metadata, "nfo:pageCount");
 	tracker_sparql_builder_object_int64 (metadata, document->getNumPages());
 
-	fts_config = tracker_main_get_fts_config ();
-	n_words = tracker_fts_config_get_max_words_to_index (fts_config);
-	content = extract_content (document, n_words);
+	config = tracker_main_get_config ();
+	n_bytes = tracker_config_get_max_bytes (config);
+	content = extract_content (document, n_bytes);
 
 	if (content) {
 		tracker_sparql_builder_predicate (metadata, "nie:plainTextContent");
